import logging
import os
import re
import subprocess

from uttl.buildout.install_recipe import InstallRecipe
from zc.buildout import UserError

class CmakeRecipe(InstallRecipe):
	def __init__(self, buildout, name, options):
		super().__init__(buildout, name, options, executable='cmake')

		self.args = [ ]

		# source

		if 'source-path' in self.options:
			self.args += [ '-S', os.path.abspath(self.options['source-path']) ]

		# generator

		if 'generator' in self.options:
			self.args += [ '-G', self.options['generator'] ]

		# configure or build

		if 'configure-path' in self.options:
			if not 'generator' in self.options:
				raise UserError('Missing mandatory "generator" option.')

			if not 'source-path' in self.options:
				raise UserError('Missing mandatory "source-path" option.')

			self.args += [ '-B', os.path.abspath(self.options['configure-path']) ]
		else:
			if not 'build-path' in self.options:
				raise UserError('Missing mandatory "build-path" option.')

			build_path = os.path.abspath(self.options['build-path'])
			self.args += [ '--build', build_path ]

			if 'target' in self.options:
				self.args += [ '--target', self.options['target'] ]
			elif 'targets' in self.options:
				targets = self.options['targets'].splitlines()
				self.args += [ '--target', ' '.join(str(t) for t in targets) ]

			if 'config' in self.options:
				self.args += [ '--config', self.options['config'] ]

		self.options['args'] = ' '.join(str(e) for e in self.args)

		# variables

		self.var_args = []

		if 'install-path' in self.options:
			install_path = os.path.abspath(self.options['install-path'])
			self.options['var-CMAKE_INSTALL_PREFIX'] = os.path.abspath(install_path) + ':PATH'

		split_name = re.compile(r'var-(.+)')
		split_type = re.compile(r'(.+):(\w*)$')

		for var in [var for var in list(self.options.keys()) if var.startswith('var-')]:
			# get name

			match = split_name.match(var)
			if not match:
				raise UserError('Failed to split variable name for "%s".' % (var))

			var_name = match.group(1)

			# get type and value

			var_value = self.options[var]

			match = split_type.match(var_value)
			if match:
				var_value = match.group(1)
				var_type = match.group(2)
			else:
				var_value = var_value
				var_type = 'STRING'

			if not any(var_type in t for t in ['BOOL', 'FILEPATH', 'PATH', 'STRING', 'INTERNAL']):
				raise UserError('Invalid variable type "%s" for "%s".' % (var_type, var))

			self.var_args += [ '-D%s:%s=%s' % (var_name, var_type, var_value) ]

		if len(self.var_args) > 0:
			if not 'generator' in self.options:
				raise UserError('Missing mandatory "generator" parameter.')

			self.var_args += [ '-G', self.options['generator'] ]

			if 'install-path' in self.options:
				self.var_args += [ '-S', os.path.abspath(self.options['install-path']) ]
			elif 'source-path' in self.options:
				self.var_args += [ '-S', os.path.abspath(self.options['source-path']) ]
			else:
				raise UserError('Missing either "install-path" or "source-path" option.')

	def install(self):
		self.working_dir = os.getcwd()
		configure_path = None

		# change to configure path

		if 'configure-path' in self.options:
			configure_path = os.path.abspath(self.options['configure-path'])

			if not os.path.exists(configure_path):
				os.makedirs(configure_path, 0o777, True)

			os.chdir(configure_path)

		# set variables

		if len(self.var_args) > 0:
			self.runCommand(self.var_args, parseLine=self.parseLine, quiet=True)

		# run command

		self.runCommand(self.args, parseLine=self.parseLine)

		# back to working directory

		if configure_path:
			os.chdir(self.working_dir)

		# add manual artefact (e.g. generated solution)

		if 'artefact-path' in self.options:
			self.options.created(os.path.abspath(self.options['artefact-path']))

		return self.options.created()

	check_errors = re.compile(r'.*Error: (.*)')
	check_failed = re.compile(r'.*(Build FAILED|CMake Error|MSBUILD : error).*')
	check_artefacts = re.compile(r'.*(.+?) -> (.+)')
	check_installed = re.compile(r'.*-- (.+?): (.+)')

	def parseLine(self, line):
		# check for errors

		if self.check_errors.match(line) or self.check_failed.match(line):
			return False

		# add artefacts to options

		match = self.check_artefacts.match(line)
		if match:
			path = match.group(2)
			self.options.created(os.path.abspath(path))

		# add installed files to options

		match = self.check_installed.match(line)
		if match:
			what = match.group(1)
			path = match.group(2)

			if any(what in s for s in ['Installing', 'Up-to-date']):
				self.options.created(os.path.abspath(path))

		return True

def uninstall(name, options):
	pass